package org.rsg.carnivore.ilan;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;

public class DiskCache {
	public static final long MAX_SIZE_DEFAULT = 500000; //bytes
	public static final String PATH_TO_CACHE = "cache/";
	public static File cachedir = new File(PATH_TO_CACHE);
	
	private static ArrayList<String> cachedFiles = new ArrayList<String>();
	public static long maximumSize = MAX_SIZE_DEFAULT;
	
	public static void main(String[] args) {
		printAll();	
		getDiskUsage();
//		deleteAll();
	}
	
	private static void mkdirCache() {
		if(!cachedir.exists())
			cachedir.mkdir();		
	}
	
	public static void writeFile(byte[] data, String fileName) {
		mkdirCache();

		fileName = PATH_TO_CACHE + fileName;
//		System.out.println("[DiskCache] writeFile: " + fileName);
		if(null==data || data.length<1) return;
		OutputStream out;
		try {
			out = new FileOutputStream(fileName);
			out.write(data);
			out.close();
			add(fileName);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		trimToSizeLimit();
	}
	
	private static void add(String fileName) {
		cachedFiles.add(fileName);
	}
	
	private static void trimToSizeLimit() {
		while(isOverSizeLimit()) 
			deleteOldestFile();
	}
	
	private static void deleteOldestFile() {
		if(cachedFiles.size() < 1) return;
		File file = new File(cachedFiles.remove(0));
		file.delete();
	}

	public static void printAll() {
		System.out.println("[DiskCache] printAll -- " + getFileCount() + " files");
		File[] files = getFiles();
		if(null==files) return;
		for(File file : files) 
			System.out.println("\t"+file);
	}

	public static void deleteAll() {
		System.out.println("[DiskCache] deleteAll -- " + getFileCount() + " files");
		File[] files = getFiles();
		if(null==files) return;
		for(File file : files) 
			file.delete();
		cachedir.delete();
//		System.out.println("[DiskCache] deleteAll -- " + getFileCount() + " files");
	}
	
	private static boolean isOverSizeLimit() {
		return getDiskUsage() > maximumSize;
	}

	public static long getDiskUsage() {
		File[] files = getFiles();
		long bytes = 0; 
		if(null==files) return 0;
		for(File file : files) 
			bytes += file.length();
//		System.out.println("[DiskCache] getDiskUsage -- "+bytes+" bytes (" + getFileCount() + " files)");
		return bytes;
	}

	public static int getFileCount() {
		File[] files = getFiles();
		if(null==files) return 0; 
		return files.length;
	}

	private static File[] getFiles() {
		return cachedir.listFiles();
	}

}
