package org.rsg.carnivore.ilan.link;

import java.util.ArrayList;
import org.rsg.carnivore.CarnivorePacket;
import org.rsg.carnivore.ilan.Node;
import org.rsg.carnivore.ilan.PortStats;

public class Link {
	  public Node origin;
	  public Node destination;
	  public PortStats portstats = new PortStats();
	  public LinkDrawer linkdrawer; 
	  public ArrayList<Particle> particles = new ArrayList<Particle>();    // An arraylist for all the particles

	  public Link(Node origin, Node destination) {
	    this.origin = origin;
	    this.destination = destination;
	    this.linkdrawer = new LinkDrawer(origin, destination);
	    //println("[Link] new link: "+this);
	  }
	  
	  public void addParticle(boolean isReversed) {
		    particles.add(new Particle(isReversed));
	  }

	  public void recalculate() {
		  linkdrawer.recalculate(origin, destination);
	  }

	  public void newPacket(CarnivorePacket packet) {
	    this.portstats.newPacket(packet);
	    
//	    System.out.println("[Link] origin.ip:" + origin.ip + " destination.ip:" + destination.ip + " packet.senderAddress:"+ packet.senderAddress);
	    
	    addParticle(origin.ip.toString().equals(packet.senderAddress.toString()) ? true : false);  
	  }

	  public void shrink() {
	    portstats.shrink();
	  }

	  public void display() {  
		  run();  
		  linkdrawer.draw(portstats.getMass());
	  }
	  
	  public void run() {
		    // Cycle through the ArrayList backwards b/c we are deleting
		    for (int i = particles.size()-1; i >= 0; i--) {
		      Particle p = (Particle) particles.get(i);
		      if(p==null) continue;
		      if (p.isDead) {
		        particles.remove(i);
		      }
		      p.run(this.linkdrawer.getPoints());
		    }
	  }

	  public String toString() {
	    return "(Link) " + origin.ip + (origin.isOnline() ? "" : " (offline)") + " -> "+destination.ip + (destination.isOnline() ? "" : " (offline)");  
	  }
}