package org.rsg.lib.chrome.button;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import org.rsg.interfascia_v2.Font;
import org.rsg.lib.chrome.Chrome;
import org.rsg.lib.chrome.Tooltip;

import processing.core.PApplet;
import processing.core.PConstants;

public abstract class AbstractButton implements MouseListener {
	protected boolean initialized = false; 
	public boolean isPicked;
	protected static final Dimension SIZE = new Dimension(13, 12);
	protected static final int MARGIN_TOP = (int) (Chrome.HEIGHT-SIZE.height)/2;
	protected static final int MARGIN_BOTTOM = MARGIN_TOP;
	protected static final int SPACER_BETWEEN = 6;
	protected static final int SPACER_RIGHT = 3;
	protected static final int CHROME_HEIGHT = 3; 
	private static final int PICKING_BORDER = 2;
	//x is set by subclasses
	protected final static int y = MARGIN_TOP; 
	protected final static int w = SIZE.width;
	protected final static int h = SIZE.height;
//	public boolean isHighlighted = false; 
	protected String tooltip = "yo";
	public static ArrayList<AbstractButton> buttons = new ArrayList<AbstractButton>();
	private int textwidth;

	//CONSTRUCTOR
	public AbstractButton() {
		super();
		buttons.add(this); //remember all buttons
	}	

	private void initWithParent(PApplet papplet) {
		initialized = true;
		Chrome.papplet.addMouseListener(this);
		Font.setSize(papplet, Font.SMALL);
		textwidth = (int) papplet.textWidth(getTooltip());		
	}
	
	//STATIC 
	public static final boolean anyButtonIsPicked() {
		for(int i = 0; i < buttons.size(); i++) {
			if(buttons.get(i).isPicked) { return true; }
		}
		return false;
	}
	
	//ABSTRACT
	public abstract Rectangle bounds();
	public abstract void click(PApplet papplet);
	public abstract String getTooltip();

	//PUBLIC
	public void draw(PApplet papplet) {
		papplet.strokeWeight(1f);
		if(!initialized) { initWithParent(papplet); }
		if(isPicked) { drawPickedBox(papplet); }
		if(Tooltip.shouldShow() && isPicked) { drawTooltip(papplet); }		
	}

	//override for buttons w/out tooltips
	public boolean pick(PApplet papplet) {
		if(isPicked(papplet.mouseX, papplet.mouseY)) {
			isPicked = true;			
		} else {
			isPicked = false;			
		}
		return isPicked;
	}

	//FINAL	
	public final Rectangle bounds4picking() {
		Rectangle r = bounds();
		return new Rectangle(r.x - PICKING_BORDER,
							 r.y - PICKING_BORDER,
							 r.width + PICKING_BORDER*2,
							 r.height + PICKING_BORDER*2);
	}
	
//	private final boolean shouldShowTooltip() {
//		return (tooltip_counter > tooltip_timeout) && (tooltip != null);
//	}

	protected final void drawPickedBox(PApplet papplet) {
		//white picked perimeter box
		papplet.fill(Chrome.white());
		papplet.stroke(Chrome.lightgray());
		papplet.rect(bounds4picking().x, bounds4picking().y, bounds4picking().width, bounds4picking().height);			
	}

	protected final void drawTooltip(PApplet papplet) {
		String s = getTooltip();
		if(s != null) {
			int w = textwidth + Font.LEADING * 2;
			int h = Font.SMALL + Font.LEADING/2;
			int x = bounds().x + bounds().width - w;
			int y = bounds().y + bounds().height + Font.LEADING*2;

			//need special coords for resize button
			if(this instanceof Resize) { 
				x = bounds().x - w;
				y = bounds().y - bounds().height - Font.LEADING;				
			}

//			//shadow 
//			papplet.fill(new Color(50, 50, 50, 40).getRGB());
//			papplet.noStroke();
//			papplet.rect(x + 3, y + 3, w, h);			

			//box 
			papplet.fill(Chrome.lightYellow());
			papplet.stroke(Chrome.gray());
			papplet.strokeWeight(.5f);
			papplet.rect(x, y, w, h);			
			
			//text
			papplet.fill(Chrome.black());
			papplet.noStroke();
			Font.setSize(papplet, Font.SMALL);
			papplet.textAlign(PConstants.LEFT);
			papplet.text(s, x + Font.LEADING, y + h - Font.LEADING/2);	
		}
	}

	public String getToolTipText(MouseEvent e) {
		System.out.println("[AbstractButton] getToolTipText("+e+")");
		return "";
	}
	
	protected final void drawBoundingBox(PApplet papplet) {
		//black box
		papplet.noFill();
		papplet.stroke(Chrome.black());
		papplet.rect(bounds().x, bounds().y, bounds().width, bounds().height);			
	}
	
	public final boolean isPicked(int x, int y) {
		return bounds4picking().contains(x,y);
	}

	public final boolean isPicked(MouseEvent e) {
		return bounds4picking().contains(e.getX(),e.getY());
	}

	//////////////////////////////////////////////////////////////////
	//INTERFACE: MouseListener
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
		if((e.getID() == MouseEvent.MOUSE_CLICKED) && isPicked(e)) {
			System.out.println("[AbstractButton] mouseClicked("+e+")");
			click(Chrome.papplet);
		}		
	}
}
