package org.rsg.lib.swing;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JProgressBar;

/**
 * A {@link JButton} that can show a {@link JProgressBar} inside it (hiding the text and icon).
 *
 * @author Chris Callendar
 * @date 15-Sep-06
 */
public class ProgressButton extends JButton {

	protected final int MIN = 0;
	protected final int MAX = 20;
	private long sleep = 500;
	private ProgressRunnable runnable = null;
	private JProgressBar bar;
	
	class ProgressRunnable extends Thread {
		private boolean running = false;
		public void run() {
			// already running?
			if (running)
				return;
			
			bar.setVisible(true);
			
			String text = getText();
			Icon icon = getIcon();
			setText(null);
			setIcon(null);
			
			running = true;
			while (running) {
//				bar.setValue((bar.getValue() + 1) % bar.getMaximum());
				bar.repaint();
				try {
					Thread.sleep(sleep);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			bar.setVisible(false);
			setText(text);
			setIcon(icon);
		}
		
		public synchronized void stopRunning() {
			running = false;
		}
	}
	
	public void doClick() {
		System.out.println("[ProgressButton] doClick()");
	}

	
	public ProgressButton() {
		super();
		initialize();
	}

//	public ProgressButton(Action a) {
//		super(a);
//		initialize();
//	}
//
//	public ProgressButton(String text) {
//		super(text);
//		initialize();
//	}
//
//	public ProgressButton(Icon icon) {
//		super(icon);
//		initialize();
//	}
//
//	public ProgressButton(String text, Icon icon) {
//		super(text, icon);
//		initialize();
//	}

	private void initialize() {
		setLayout(new BorderLayout(0, 0));		
		bar = new JProgressBar(MIN, MAX);
		bar.setIndeterminate(true);		
		bar.setVisible(false);
		bar.setStringPainted(false);
//		bar.setBackground(Color.white);
//		bar.setForeground(Color.green);
		add(bar, BorderLayout.CENTER);
	}
	
	/**
	 * Sets the speed.
	 * @param speed an int from 1 - 10, 10 is fastest
	 */
	public void setSpeed(int speed) {
		int speedInverse = 11 - Math.max(1, Math.min(10, speed));	// from 1 - 10
		sleep = speedInverse * 10L;
	}
	
	/**
	 * Sets the foreground and background colors for the progress bar.
	 * @param foreground
	 * @param background
	 */
	public void setProgressColors(Color foreground, Color background) {
		bar.setForeground(foreground);
		bar.setBackground(background);
	}
	
	/**
	 * Makes the progress bar visible.
	 */
	public void showProgressBar() {
		runnable = new ProgressRunnable();
		runnable.start();
	}

	
	/**
	 * Hides the progress bar.
	 */
	public void hideProgressBar() {
		if (runnable != null) {
			runnable.stopRunning();
		}
	}
	
//	public static void main(String[] args) {
//		JDialog dlg = new JDialog();
//		dlg.setModal(true);
//		dlg.setTitle("Test");
//		
//		JPanel pnl = new JPanel();
//		pnl.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
//
//		final ProgressButton btn = new ProgressButton();
//		Action action = new AbstractAction("Start") {
//			public void actionPerformed(ActionEvent e) {
//				btn.showProgressBar();
//				new Thread(new Runnable() {
//					public void run() {
//						try {
//							Thread.sleep(3000);
//						} catch (InterruptedException e1) {
//						}
//						btn.hideProgressBar();
//					}
//				}).start();
//			}
//		};
//		btn.setAction(action);
//		btn.setPreferredSize(new Dimension(200, 24));
//		pnl.add(btn);
//		dlg.getContentPane().add(pnl, BorderLayout.CENTER);
//		
//		dlg.setSize(550, 150);
//		dlg.setVisible(true);
//		System.exit(0);
//		
//	}

}